from importlib import import_module
from typing import Union, List

mapper = {
    # Trees
    "dtc": ["sklearn.tree", "DecisionTreeClassifier"],
    "dtr": ["sklearn.tree", "DecisionTreeRegressor"],
    "etc": ["sklearn.tree", "ExtraTreeClassifier"],
    "etr": ["sklearn.tree", "ExtraTreeRegressor"],

    # SVM
    "svc": ["sklearn.svm", "LinearSVC"],
    "svr": ["sklearn.svm", "LinearSVR"],

    # Clustering
    "kmeans": ["sklearn.cluster", "KMeans"],

    # Ensemble
    "abc": ["sklearn.ensemble", "AdaBoostClassifier"],
    "abr": ["sklearn.ensemble", "AdaBoostRegressor"],
    "bc": ["sklearn.ensemble", "BaggingClassifier"],
    "br": ["sklearn.ensemble", "BaggingRegressor"],
    "etc": ["sklearn.ensemble", "ExtraTreesClassifier"],
    "etr": ["sklearn.ensemble", "ExtraTreesRegressor"],
    "gbc": ["sklearn.ensemble", "GradientBoostingClassifier"],
    "gbr": ["sklearn.ensemble", "GradientBoostingRegressor"],
    "rfc": ["sklearn.ensemble", "RandomForestClassifier"],
    "rfr": ["sklearn.ensemble", "RandomForestRegressor"],
}

unsupervised = set(["sklearn.cluster.KMeans"])

"""
Problem: When experimenting with models, and you just want to train something right away,
just to see how a model works right out of the box without any HP tuning, you need to do 3
statements: import, initialise and fit.

Also, when trying to fit multiple models, the import statement does get bigger
and more boilerplate is added, so this is a simple way of eliminating that
"""
def get_module(base:str, module:str):
        return getattr(import_module(base), module) 

def fit_one_estimator(abbr: str, X, y = None):
    cls, e = mapper[abbr]

    if f"{cls}.{e}" in unsupervised:
        assert y is None, "For unsupervised techniques, y must not exist"
    else:
        assert X is not None and y is not None, "X and y must exist"

    estimator = get_module(cls, e)

    if y is None:
        return estimator().fit(X)
    else:
        return estimator().fit(X, y)

def fit_estimators(abbr: Union[str, List[str]], X, y = None):
    if isinstance(abbr, str):
        assert abbr in mapper, "The abbr must be one of: {}".format(list(mapper.keys()))
        return fit_one_estimator(abbr, X, y)
    
    return list(map(lambda x: fit_one_estimator(x, X, y), abbr))


    



import os
import pandas as pd

def make_dataframe(path: str, **kwargs) -> pd.DataFrame:
    ext = os.path.splitext(path)[1][1::]
    if ext == "csv":
        fn = pd.read_csv
    elif ext in ("xls", "xlsx"):
        fn = pd.read_excel
    elif ext in ("pkl", "pickle"):
        fn = pd.read_parquet
    else:
        raise ValueError("Currently only csv, xls, xlsx, pkl and pickle files are supported")
    return fn(path, **kwargs)    



import os.path as path
from typing import Callable, Dict, Any

import pandas as pd

from .make_dataframe import make_dataframe

def get_dataloader(prefix: str) -> Callable[[str, Dict[str, Any]], pd.DataFrame]:
    def dataloader(name: str, **kwargs) -> pd.DataFrame:
        file_path = path.join(prefix, name)
        return make_dataframe(file_path, **kwargs)
    return dataloader


import pathlib
from setuptools import setup, find_packages

import abuu

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="abuu",
    version=abuu.__version__,
    description="ABUU - Another Bunch of Usefull Utils",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/yashshah1/abuu",
    author="Yash Shah",
    author_email="yashshah1234@gmail.com",
    license="MIT",
    classifiers=[
        'Development Status :: 3 - Alpha',
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
    	"pandas>=1.0.0",
    	"scikit-learn>=0.22.1"
    ],
)



from typing import Dict, Generic, TypeVar, Union
import json

V = TypeVar("V")

class ConfigClass(Generic[V]):
    """
    A lot of config usually exists as JSON objects, when parsed in Python,
    it results in a dict being generated which can be accessed as dict['key']

    However, this looks less cleaner than writing config.key, which is more elegant.
    This function aims to convert a given config dict into a class which can be accessed as above.
    """

    _data: Dict[str, V]

    def __init__(self, data: Union[str, Dict[str, V]]) -> None:
        """Can accept a dict or json string"""
        # we dont check that the keys are strings for 2 reasons
        # 1) it is shown in the type hints that it must be a string, so users should make sure of that them self.
        # 2) this is expected to take json, and json always has str keys
        if isinstance(data, str):
            self._data = json.loads(data)
        elif isinstance(data, dict):
            self._data = data
        else:
            raise ValueError("data must be either a JSON string or a python dict")

    def __getattr__(self, key: str) -> V:
        """Try and get the attribute from data"""
        try:
            return self._data[key]
        except KeyError as e:
            raise AttributeError(f"no attribute {key}.") from e



get_config_class = ConfigClass  # for backwards compatibility.
